import os,re

class bob():

    @staticmethod
    def postgisToSpatialite(sql, targetSrid='2154'):
        '''
        Convert postgis SQL statement
        into spatialite compatible
        statements
        '''

        # delete some incompatible options
        # replace other by spatialite syntax
        replaceDict = [
            # delete
            {'in': r'with\(oids=.+\)', 'out': ''},
            {'in': r'comment on [^;]+;', 'out': ''},
            {'in': r'alter table ([^;]+) add primary key( )+\(([^;]+)\);',
            'out': r'create index idx_\1_\3 on \1 (\3);'},
            {'in': r'alter table ([^;]+) add constraint [^;]+ primary key( )+\(([^;]+)\);',
            'out': r'create index idx_\1_\3 on \1 (\3);'},
            {'in': r'alter table [^;]+drop column[^;]+;', 'out': ''},
            {'in': r'alter table [^;]+drop constraint[^;]+;', 'out': ''},
            #~ {'in': r'^analyse [^;]+;', 'out': ''},
            # replace
            {'in': r'truncate (bati|fanr|lloc|nbat|pdll|prop)',
            'out': r'drop table \1;create table \1 (tmp text)'},
            {'in': r'truncate ', 'out': 'delete from '},
            {'in': r'distinct on *\([a-z, ]+\)', 'out': 'distinct'},
            {'in': r'serial', 'out': 'INTEGER PRIMARY KEY AUTOINCREMENT'},
            {'in': r'string_agg', 'out': 'group_concat'},
            {'in': r'current_schema::text, ', 'out': ''},
            {'in': r'substring', 'out': 'SUBSTR'},
            {'in': r"(to_char\()([^']+) *, *'[09]+' *\)", 'out': r"CAST(\2 AS TEXT)"},
            {'in': r"(to_number\()([^']+) *, *'[09]+' *\)", 'out': r"CAST(\2 AS float)"},
            {'in': r"(to_date\()([^DD]+) *, *'DDMMYYYY' *\)",
            'out': r"date(substr(\2, 5, 4) || '-' || substr(\2, 3, 2) || '-' || substr(\2, 1, 2))"},
            {'in': r"(to_date\()([^DD]+) *, *'DD/MM/YYYY' *\)",
            'out': r"date(substr(\2, 7, 4) || '-' || substr(\2, 4, 2) || '-' || substr(\2, 1, 2))"},
            {'in': r"(to_date\()([^DD]+) *, *'YYYYMMDD' *\)",
            'out': r"date(substr(\2, 1, 4) || '-' || substr(\2, 5, 2) || '-' || substr(\2, 7, 2))"},
            {'in': r"(to_char\()([^']+) *, *'dd/mm/YYYY' *\)",
            'out': r"strftime('%d/%m/%Y', \2)"},
            {'in': r"\)\+\((SUBSTR\(tmp\,[0-9]+\,[0-9]+\))::integer - 1 \|\| ' day'\)::interval",
            'out': r", '+' || cast(\1-1 as text)  || ' day')"},
            {'in': r"ST_MakeValid\(geom\)",
             'out': r"CASE WHEN ST_IsValid(geom) THEN geom ELSE ST_Buffer(geom,0) END"},
            {'in': r"ST_MakeValid\(p\.geom\)",
             'out': r"CASE WHEN ST_IsValid(p.geom) THEN p.geom ELSE ST_Buffer(p.geom,0) END"},
            {'in': r' ~ ', 'out': ' regexp '}
        ]

        for a in replaceDict:
            r = re.compile(a['in'], re.IGNORECASE|re.MULTILINE)
            sql = r.sub(a['out'], sql)

        # index spatiaux
        r = re.compile(r'(create index [^;]+ ON )([^;]+)( USING +)(gist +)?\(([^;]+)\);',  re.IGNORECASE|re.MULTILINE)
        sql = r.sub(r"SELECT createSpatialIndex('\2', '\5');", sql)

        # replace postgresql "update from" statement
        r = re.compile(r'(update [^;=]+)(=)([^;=]+ FROM [^;]+)(;)', re.IGNORECASE|re.MULTILINE)
        sql = r.sub(r'\1=(SELECT \3);', sql)

        #self.updateLog(sql)
        return sql


sql = '''SELECT
CASE WHEN trim(SUBSTRING(tmp,72,8))='' THEN NULL ELSE to_date('01' || SUBSTRING(tmp,74,6),'DDMMYYYY')+(SUBSTRING(tmp,72,2)::integer - 1 || ' day')::interval END AS datact
FROM machin

'''

sql='''
INSERT INTO parcelle ( parcelle, annee, ccodep, ccodir, ccocom, ccopre, ccosec, dnupla, dcntpa, dsrpar, dnupro, jdatat, dreflf, gpdl, cprsecr, ccosecr, dnuplar, dnupdl, gurbpa, dparpi, ccoarp, gparnf, gparbat, parrev, gpardp, fviti, dnvoiri, dindic, ccovoi, ccoriv, ccocif, gpafpd, ajoutcoherence, comptecommunal, pdl, voie, cconvo, dvoilib, ccocomm, ccoprem, ccosecm, dnuplam, parcellefiliation, type_filiation, ccoifp, lot ) SELECT
SUBSTR(tmp,31,1) AS dsrpar, SUBSTR(tmp,32,6) AS dnupro, CASE WHEN trim(SUBSTR(tmp,38,8))='' THEN NULL ELSE date(substr('01' || SUBSTR(tmp,40,6), 5, 4) || '-' || substr('01' || SUBSTR(tmp,40,6), 3, 2) || '-' || substr('01' || SUBSTR(tmp,40,6), 1, 2))+(SUBSTR(tmp,38,2)::integer - 1 || ' day')::interval END AS jdatat, SUBSTR(tmp,46,5) AS dreflf, SUBSTR(tmp,51,1) AS gpdl, SUBSTR(tmp,52,3) AS cprsecr, SUBSTR(tmp,55,2) AS ccosecr
'''

b = bob()
print(b.postgisToSpatialite(sql))
